#!/bin/ksh
#
# Copyright (C) Tivoli Systems/IBM. <2000>
# Unpublished Work
# All Rights Reserved
# Licensed Material - Property of Tivoli Systems/IBM.
#

##############################################################################
# SCRIPT NAME: kossdump.sh
##############################################################################
#
# DESCRIPTION:
#
# This script will generate the output file specified by the third parameter,
# which will contain basic info from the system crash that occurred.  Info
# includes:
#    Name of the system, and OS level
#    The message buffer at the time of the crash
#    Stack of the asserting thread
#    A dump of all thread stacks in the system crash
#    A full stack trace of the asserting thread (with registers)
# 
##############################################################################

KERNEL=""
IMAGE=""
ISLIVE=1
PDOSBIN="/opt/pdos/bin"
KERNBIN="/opt/pdos/kernel"
PDOSINST="/opt/pdos"

##############################################################################
# FUNCTION: usage
##############################################################################
#
# This function prints the proper usage of the script.
#
##############################################################################
usage() {
  print "ERROR: Unknown option/bad syntax:"
  print "usage: kossdump.sh [-k kernel_symbols] [-d dump_image]"
  exit 1
}

##############################################################################
# FUNCTION: process_args
##############################################################################
#
# This function processes the command line args, displaying any relevant
# error messages.
#
##############################################################################
process_args () {
  while getopts ":k:d:" opt; do
    case $opt in
      k)
        ISLIVE=0
        KERNEL=$OPTARG
	;;
      d)
        ISLIVE=0
        IMAGE=$OPTARG
	;;
      \?) usage ;;
    esac
  done

  shift $(($OPTIND - 1))

  if [ -z $KERNEL ]; then
    if [ ISLIVE -eq 0 ] ; then
        print "kernel (-k) must be specified with -d"
	usage
    else
        print "DEFAULT kernel symbol table /dev/ksyms!:"
        KERNEL=/dev/ksyms
    fi
  fi
  
  if [[ -z $IMAGE ]]; then
    if [ ISLIVE -eq 0 ] ; then
        print "dump image (-d) must be specified with -k"
	usage
    else
        print "DEFAULT for system image is on live system!"
    fi
  fi

}

####################################################################
####################################################################
####################################################################
####################################################################
# Main Starting point
####################################################################

 #####  #######    #    ######  #######
#     #    #      # #   #     #    #
#          #     #   #  #     #    #
 #####     #    #     # ######     #
      #    #    ####### #   #      #
#     #    #    #     # #    #     #
 #####     #    #     # #     #    #

#     #    #      ###   #     #
##   ##   # #      #    ##    #
# # # #  #   #     #    # #   #
#  #  # #     #    #    #  #  #
#     # #######    #    #   # #
#     # #     #    #    #    ##
#     # #     #   ###   #     #

####################################################################
####################################################################
####################################################################

process_args $@

print "kossdump@`hostname`: `date`"
if [ $ISLIVE = 1 ] ; then
    print "  Info is from running system: `hostname`"

    print "====================================================="
    print "====================================================="
    print "Kosseal kernel trace data:"
    print "====================================================="
    ${KERNBIN}/kazntrace -l
    ${KERNBIN}/kazntrace

    # get statistics
    print "====================================================="
    print "====================================================="
    print "Kosseal kernel statistics data"
    print "====================================================="
    ${KERNBIN}/kossinfo
    print ""
    ${KERNBIN}/kossinfo -x

else
    print "  Info is from system dump image $IMAGE, unix: $KERNEL"
fi
print ""

adb -k $KERNEL $IMAGE << EOF
="Initial Dump Information"
="========================"
$<<utsname
srpc_domain/s15t"Domain name"
time/Y15t"Time of crash"
lbolt>a
*time-(*<a%0t100)=Y15t"Time of boot"
*panicstr/n"Panic string:"ts
=n"Message Buffer:"
$<<msgbuf
=n"Stack traceback"
=n
$ C
=n"All running threads:"
$<<threadlist
=n"Full stack traceback with registers:"
<sp$<<stacktrace
=n"VFS info:"
$<<mount
$<<vfslist
=n"Kernel Memory info:"
$<<kmastat
=n"ooooooooooooo PDOS internal data oooooooooooooo"
=n"kazn stats:"
kaznStats/30D
=n"kpcfile stats:"
kpcFileStats/40D
=n"kazn rc stats:"
kazn_rcfStats/12D
kazn_rcIdStats/8D
=n"pal memStats:"
pal_memStats/10D
pal_sysMemMB/1D
=n"kenv proc stats:"
kenv_procStats/4D
=n"kaud info:"
kaud_audit_level/1X
kaud_warning_mode/1X
kaud_intTraceFlags/1X
=n"Misc info:"
kutil_kossActive/1X
kutil_kossWasActive/1X
kenv_procRegisteredProcs/1D
kutil_privUid/2D
kutil_kossDebugFlag/1D
kazn_remappedTimeouts/1D
=n"TLI info:"
koss_tliStats/28D
EOF

#
print "End of adb gathered data"
print "====================================================="

#
# Data from the kosseal error logs
#
print "====================================================="
print "====================================================="
print "\nkosserrs.log:"
$KERNBIN/kosserrs
#

if [ -s /var/pdos/log/kosserrs.log.old ] ; then
    print "\nprevious kosserrs.log"
    $KERNBIN/kosserrs -f /var/pdos/log/kosserrs.log.old
fi

#                                                          #
############################################################
# Code added for Defect 2974.                              #
############################################################

############################################################
# Check to see what the kernel symbol table specified was. #
# If it was /dev/ksyms, then running on a live system and  #
# gather live sys info.                                    #
############################################################
if [[ -z $IMAGE || $IMAGE == "/dev/ksyms" ]]; then
    print "GATHER live system data."
    print "\n\n"
    print "====================================================="
    print "====================================================="
    print "Current running processes (ps -elf)"
    print "====================================================="
    /usr/bin/ps -elf | sort -n +3
    print "====================================================="
    print "====================================================="
    print "Loaded drivers:"
    modinfo
    print "====================================================="
fi

print "\n\n====================================================="
print "====================================================="
print "System and software info"
print "====================================================="
showrev -a
print -n "System conf: \n"
prtconf -P
print -n "System definition: \n"
sysdef -D $KERNEL
print "Network configuration:"
print "\tnetstat -in"
netstat -in
print "\tnetstat -s"
netstat -s
print "\tifconfig"
ifconfig -a
print "\nSystem (/etc/system) configuration parameters:"
cat /etc/system | egrep -v  ^[*] | egrep [a-zA-Z]
print ""

print "\n\n====================================================="
print "====================================================="
print "/var/adm/messages data"
print "====================================================="
tail -200 /var/adm/messages
print ""

print "\n\n====================================================="
print "s/w version data"
OSLEVEL=`uname -r`
if [[ $OSLEVEL == "5.7" || $OSLEVEL == "5.8" ]] ; then
    # Check the arch support
    isainfo | grep sparcv9 > /dev/null
    if [ $? -eq 0 ] ; then
	ARCH="sparcv9/"
	if [[ $OSLEVEL == "5.7" ]] ; then
	    BITS=64_27
	else
	    BITS=64_28
	fi
    else
	if [[ $OSLEVEL == "5.7" ]] ; then
	    BITS=32_27
	else
	    BITS=32_28
	fi
    fi
else
    BITS=32_26
fi
print -n "/usr/kernel/drv/${ARCH}kazndrv sum: "
print "`sum  ${PDOSINST}/kernel/kazndrv`"
print -n "/usr/kernel/strmod/${ARCH}kazntli sum: "
print "`sum ${PDOSINST}/kernel/kazntli`"
print -n "${PDOSBIN}/pdosd: "
print "`sum ${PDOSBIN}/pdosd`"
if [ -s /usr/ccs/bin/what ] ; then
    /usr/ccs/bin/what ${PDOSINST}/kernel/kazndrv${BITS}
    /usr/ccs/bin/what ${PDOSINST}/kernel/kazntli${BITS}
    /usr/ccs/bin/what ${PDOSBIN}/pdosd
fi

print "\n\n"
print "ooooooooooooooooooooooooooooooooooooooooooooooooooooo"
print "PDOS configuration info:\n"

ls -l /opt/pdos/etc

print "\n\n"
print "osseal.conf:"
print "oooooooooooo"
cat /opt/pdos/etc/osseal.conf

print "\n\n"
print "pdosd.conf"
print "oooooooooo"
cat /opt/pdos/etc/pdosd.conf


print "\n\n"
print "lpm.conf"
print "oooooooo"
cat /opt/pdos/etc/lpm.conf

print "\n\n"
print "/var/pdos dir contents (ls -lsR)"
print "ooooooooooooooooooooooooooooooooo"
ls -lsR /var/pdos

print "\n\n"
print "Free space in major PDOS data directories"
print "ooooooooooooooooooooooooooooooooooooooooo"
print "/var/pdos/azn:"
df -k /var/pdos/azn
print "/var/pdos/audit:"
df -k /var/pdos/audit
print "/var/pdos/cred:"
df -k /var/pdos/cred
print "/var/pdos/log:"
df -k /var/pdos/log

print "\n\n"
print " -- PDOS log files --"
/usr/bin/ls -l /var/pdos/log

print "pdosd log file data"
print "ooooooooooooooooooo"
tail -200 /var/pdos/log/pdosd.log
tail -200 /var/pdos/log/pdosd.log.* 2> /dev/null

print "\n\n"
print "pdosauditd log file data"
print "oooooooooooooooooooooooo"
tail -100 /var/pdos/log/pdosauditd.log
tail -100 /var/pdos/log/pdosauditd.log.* 2> /dev/null

print "\n\n"
print "pdoswdd log file data"
print "oooooooooooooooooooooo"
tail -100 /var/pdos/log/pdoswdd.log
tail -100 /var/pdos/log/pdoswdd.log.* 2> /dev/null

print "oooooooooooooooo ---> End of Collected Data <--- ooooooooooooooo"

exit 0
